@@ -38,6 +38,8 @@
 #define TOGGLE_ANIMATION_BUTTON 4
 #define STEP_ANIMATION_BUTTON 5
 #undef ALWAYS_LOAD_ORIGINAL_SIZE
+#define N_FORWARD_PRELOADERS 2
+#define N_BACKWARD_PRELOADERS 2
 
 
 static void gth_viewer_page_interface_init (GthViewerPageInterface *iface);
@@ -95,6 +97,7 @@ struct _GthImageViewerPagePrivate {
 	guint              file_popup_merge_id;
 	GthImageHistory   *history;
 	GthFileData       *file_data;
+	GFileInfo         *updated_info;
 	gboolean           active;
 	gboolean           image_changed;
 	gboolean           loading_image;
@@ -108,6 +111,8 @@ struct _GthImageViewerPagePrivate {
 	gboolean           pointer_on_overview;
 	guint              hide_overview_id;
 	gboolean           apply_icc_profile;
+	GthFileData       *next_file_data[N_FORWARD_PRELOADERS];
+	GthFileData       *prev_file_data[N_BACKWARD_PRELOADERS];
 };
 
 
@@ -123,6 +128,7 @@ gth_image_viewer_page_file_loaded (GthImageViewerPage *self,
 
 	gth_viewer_page_file_loaded (GTH_VIEWER_PAGE (self),
 				     self->priv->file_data,
+				     self->priv->updated_info,
 				     success);
 }
 
@@ -199,6 +205,126 @@ _gth_image_preloader_get_requested_size_for_current_image (GthImageViewerPage *s
 }
 
 
+/* -- _gth_image_viewer_page_load_with_preloader -- */
+
+
+typedef struct {
+	GthImageViewerPage  *self;
+	GthFileData         *file_data;
+	int                  requested_size;
+	GCancellable        *cancellable;
+	GAsyncReadyCallback  callback;
+	gpointer	     user_data;
+} ProfileData;
+
+
+static void
+profile_data_free (ProfileData *profile_data)
+{
+	_g_object_unref (profile_data->cancellable);
+	_g_object_unref (profile_data->file_data);
+	_g_object_unref (profile_data->self);
+	g_free (profile_data);
+}
+
+
+static void
+_gth_image_viewer_page_load_with_preloader_step2 (GthImageViewerPage  *self,
+						  GthFileData         *file_data,
+						  int                  requested_size,
+						  GCancellable        *cancellable,
+						  GAsyncReadyCallback  callback,
+						  gpointer	       user_data)
+{
+	g_object_ref (self);
+
+	gth_image_preloader_load (self->priv->preloader,
+				  file_data,
+				  requested_size,
+				  cancellable,
+				  callback,
+				  user_data,
+				  N_FORWARD_PRELOADERS + N_BACKWARD_PRELOADERS,
+				  self->priv->next_file_data[0],
+				  self->priv->next_file_data[1],
+				  self->priv->prev_file_data[0],
+				  self->priv->prev_file_data[1]);
+}
+
+
+static void
+profile_ready_cb (GObject      *source_object,
+                  GAsyncResult *res,
+                  gpointer      user_data)
+{
+	ProfileData        *profile_data = user_data;
+	GthImageViewerPage *self = profile_data->self;
+	GthICCProfile      *profile;
+
+	profile = gth_color_manager_get_profile_finish (GTH_COLOR_MANAGER (source_object), res, NULL);
+	if (profile == NULL)
+		profile = _g_object_ref (gth_browser_get_monitor_profile (self->priv->browser));
+	gth_image_preloader_set_out_profile (self->priv->preloader, profile);
+
+	_gth_image_viewer_page_load_with_preloader_step2 (profile_data->self,
+							  profile_data->file_data,
+							  profile_data->requested_size,
+							  profile_data->cancellable,
+							  profile_data->callback,
+							  profile_data->user_data);
+
+	_g_object_unref (profile);
+	profile_data_free (profile_data);
+}
+
+
+static void
+_gth_image_viewer_page_load_with_preloader (GthImageViewerPage  *self,
+					    GthFileData         *file_data,
+					    int                  requested_size,
+					    GCancellable        *cancellable,
+					    GAsyncReadyCallback  callback,
+					    gpointer		 user_data)
+{
+	if ((file_data != NULL) && self->priv->apply_icc_profile) {
+		char *monitor_name = NULL;
+
+		if (_gtk_window_get_monitor_info (GTK_WINDOW (self->priv->browser), NULL, NULL, &monitor_name)) {
+			ProfileData *profile_data;
+
+			profile_data = g_new (ProfileData, 1);
+			profile_data->self = g_object_ref (self);
+			profile_data->file_data = g_object_ref (file_data);
+			profile_data->requested_size = requested_size;
+			profile_data->cancellable = _g_object_ref (cancellable);
+			profile_data->callback = callback;
+			profile_data->user_data = user_data;
+
+			gth_color_manager_get_profile_async (gth_main_get_default_color_manager(),
+							     monitor_name,
+							     cancellable,
+							     profile_ready_cb,
+							     profile_data);
+
+			return;
+		}
+	}
+
+	gth_image_preloader_set_out_profile (self->priv->preloader, NULL);
+	_gth_image_viewer_page_load_with_preloader_step2 (self, file_data, requested_size, cancellable, callback, user_data);
+}
+
+
+static gboolean
+_gth_image_viewer_page_load_with_preloader_finish (GthImageViewerPage  *self)
+{
+	gboolean active = self->priv->active;
+	g_object_unref (self);
+
+	return active;
+}
+
+
 static void
 different_quality_ready_cb (GObject		*source_object,
 			    GAsyncResult	*result,
@@ -216,6 +342,9 @@ different_quality_ready_cb (GObject		*source_object,
 	int                 w1, h1, w2, h2;
 	gboolean            got_better_quality;
 
+	if (! _gth_image_viewer_page_load_with_preloader_finish (self))
+		return;
+
 	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
 					       result,
 					       &requested,
@@ -287,16 +416,6 @@ _g_mime_type_can_load_different_quality (const char *mime_type)
 }
 
 
-static void
-_gth_image_preloader_init_preloader (GthImageViewerPage *self)
-{
-	if (self->priv->apply_icc_profile)
-		gth_image_preloader_set_out_profile (self->priv->preloader, gth_browser_get_screen_profile (self->priv->browser));
-	else
-		gth_image_preloader_set_out_profile (self->priv->preloader, NULL);
-}
-
-
 static gboolean
 update_quality_cb (gpointer user_data)
 {
@@ -307,21 +426,24 @@ update_quality_cb (gpointer user_data)
 		self->priv->update_quality_id = 0;
 	}
 
+	if (! self->priv->active)
+		return FALSE;
+
+	if (self->priv->viewer == NULL)
+		return FALSE;
+
 	if (self->priv->loading_image)
 		return FALSE;
 
 	if (! self->priv->image_changed && ! _g_mime_type_can_load_different_quality (gth_file_data_get_mime_type (self->priv->file_data)))
 		return FALSE;
 
-	_gth_image_preloader_init_preloader (self);
-	gth_image_preloader_load (self->priv->preloader,
-				  self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
-				  _gth_image_preloader_get_requested_size_for_current_image (self),
-				  NULL,
-				  different_quality_ready_cb,
-				  self,
-				  GTH_NO_PRELOADERS,
-				  NULL);
+	_gth_image_viewer_page_load_with_preloader (self,
+						    self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
+						    _gth_image_preloader_get_requested_size_for_current_image (self),
+						    NULL,
+						    different_quality_ready_cb,
+						    self);
 
 	return FALSE;
 }
@@ -576,15 +698,6 @@ viewer_image_map_event_cb (GtkWidget          *widget,
 }
 
 
-static gboolean
-viewer_key_press_cb (GtkWidget          *widget,
-		     GdkEventKey        *event,
-		     GthImageViewerPage *self)
-{
-	return gth_browser_viewer_key_press_cb (self->priv->browser, event);
-}
-
-
 static void
 clipboard_targets_received_cb (GtkClipboard *clipboard,
 			       GdkAtom      *atoms,
@@ -636,7 +749,7 @@ viewer_realize_cb (GtkWidget *widget,
 	GthImageViewerPage *self = user_data;
 	GtkClipboard       *clipboard;
 
-	clipboard = gtk_widget_get_clipboard (self->priv->viewer, GDK_SELECTION_CLIPBOARD);
+	clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
 	g_signal_connect (clipboard,
 	                  "owner_change",
 	                  G_CALLBACK (clipboard_owner_change_cb),
@@ -651,7 +764,7 @@ viewer_unrealize_cb (GtkWidget *widget,
 	GthImageViewerPage *self = user_data;
 	GtkClipboard       *clipboard;
 
-	clipboard = gtk_widget_get_clipboard (self->priv->viewer, GDK_SELECTION_CLIPBOARD);
+	clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
 	g_signal_handlers_disconnect_by_func (clipboard,
 	                                      G_CALLBACK (clipboard_owner_change_cb),
 	                                      self);
@@ -726,6 +839,9 @@ pref_zoom_quality_changed (GSettings *settings,
 {
 	GthImageViewerPage *self = user_data;
 
+	if (! self->priv->active || (self->priv->viewer == NULL))
+		return;
+
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer),
 					   g_settings_get_enum (self->priv->settings, PREF_IMAGE_VIEWER_ZOOM_QUALITY));
 	gtk_widget_queue_draw (self->priv->viewer);
@@ -739,6 +855,9 @@ pref_zoom_change_changed (GSettings *settings,
 {
 	GthImageViewerPage *self = user_data;
 
+	if (! self->priv->active || (self->priv->viewer == NULL))
+		return;
+
 	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->viewer),
 					  g_settings_get_enum (self->priv->settings, PREF_IMAGE_VIEWER_ZOOM_CHANGE));
 	gtk_widget_queue_draw (self->priv->viewer);
@@ -752,6 +871,9 @@ pref_reset_scrollbars_changed (GSettings *settings,
 {
 	GthImageViewerPage *self = user_data;
 
+	if (! self->priv->active || (self->priv->viewer == NULL))
+		return;
+
 	gth_image_viewer_set_reset_scrollbars (GTH_IMAGE_VIEWER (self->priv->viewer),
 					       g_settings_get_boolean (self->priv->settings, PREF_IMAGE_VIEWER_RESET_SCROLLBARS));
 }
@@ -982,6 +1104,7 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 	self->priv->preloader = gth_browser_get_image_preloader (browser);
 
 	self->priv->viewer = gth_image_viewer_new ();
+	g_object_add_weak_pointer (G_OBJECT (self->priv->viewer), (gpointer *) &self->priv->viewer);
 	gtk_widget_add_events (self->priv->viewer, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
 	gth_image_viewer_page_reset_viewer_tool (self);
 	gtk_widget_show (self->priv->viewer);
@@ -1010,10 +1133,6 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 				"map_event",
 				G_CALLBACK (viewer_image_map_event_cb),
 				self);
-	g_signal_connect (G_OBJECT (self->priv->viewer),
-			  "key_press_event",
-			  G_CALLBACK (viewer_key_press_cb),
-			  self);
 	g_signal_connect (G_OBJECT (self->priv->viewer),
 			  "realize",
 			  G_CALLBACK (viewer_realize_cb),
@@ -1142,6 +1261,8 @@ preloader_load_ready_cb (GObject	*source_object,
 	GError		   *error = NULL;
 
 	self->priv->loading_image = FALSE;
+	if (! _gth_image_viewer_page_load_with_preloader_finish (self))
+		return;
 
 	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
 					       result,
@@ -1187,13 +1308,29 @@ preloader_load_ready_cb (GObject	*source_object,
 		gth_image_viewer_get_original_size (GTH_IMAGE_VIEWER (self->priv->viewer),
 						    &original_width,
 						    &original_height);
-	g_file_info_set_attribute_int32 (self->priv->file_data->info,
+	g_file_info_set_attribute_int32 (self->priv->updated_info,
 					 "frame::width",
 					 original_width);
-	g_file_info_set_attribute_int32 (self->priv->file_data->info,
+	g_file_info_set_attribute_int32 (self->priv->updated_info,
 					 "frame::height",
 					 original_height);
 
+	{
+		GthICCProfile *profile;
+
+		profile = gth_image_get_icc_profile (image);
+		if (profile != NULL) {
+			char *desc = gth_icc_profile_get_description (profile);
+
+			if (desc != NULL) {
+				g_file_info_set_attribute_string (self->priv->updated_info,
+								  "Loaded::Image::ColorProfile",
+								  desc);
+				g_free (desc);
+			}
+		}
+	}
+
 	gth_image_viewer_page_file_loaded (self, TRUE);
 	update_image_quality_if_required (self);
 
@@ -1207,10 +1344,6 @@ clear_data:
 }
 
 
-#define N_FORWARD_PRELOADERS 2
-#define N_BACKWARD_PRELOADERS 2
-
-
 static void
 _gth_image_viewer_page_load (GthImageViewerPage *self,
 		             GthFileData        *file_data)
@@ -1218,20 +1351,20 @@ _gth_image_viewer_page_load (GthImageViewerPage *self,
 	GthFileStore *file_store;
 	GtkTreeIter   iter;
 	int           i;
-	GthFileData  *next_file_data[N_FORWARD_PRELOADERS];
-	GthFileData  *prev_file_data[N_BACKWARD_PRELOADERS];
 
 	if (self->priv->file_data != file_data) {
 		_g_object_unref (self->priv->file_data);
 		self->priv->file_data = gth_file_data_dup (file_data);
+		_g_object_unref (self->priv->updated_info);
+		self->priv->updated_info = g_file_info_new ();
 	}
 	self->priv->image_changed = FALSE;
 	self->priv->loading_image = TRUE;
 
 	for (i = 0; i < N_FORWARD_PRELOADERS; i++)
-		next_file_data[i] = NULL;
+		_g_clear_object (&self->priv->next_file_data[i]);
 	for (i = 0; i < N_BACKWARD_PRELOADERS; i++)
-		prev_file_data[i] = NULL;
+		_g_clear_object (&self->priv->prev_file_data[i]);
 
 	file_store = gth_browser_get_file_store (self->priv->browser);
 	if (gth_file_store_find_visible (file_store, self->priv->file_data->file, &iter)) {
@@ -1241,36 +1374,29 @@ _gth_image_viewer_page_load (GthImageViewerPage *self,
 		for (i = 0; i < N_FORWARD_PRELOADERS; i++) {
 			if (! gth_file_store_get_next_visible (file_store, &next_iter))
 				break;
-			next_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+			self->priv->next_file_data[i] = g_object_ref (gth_file_store_get_file (file_store, &next_iter));
 		}
 
 		next_iter = iter;
 		for (i = 0; i < N_BACKWARD_PRELOADERS; i++) {
 			if (! gth_file_store_get_prev_visible (file_store, &next_iter))
 				break;
-			prev_file_data[i] = gth_file_store_get_file (file_store, &next_iter);
+			self->priv->prev_file_data[i] = g_object_ref (gth_file_store_get_file (file_store, &next_iter));
 		}
 
 		gth_image_viewer_set_void (GTH_IMAGE_VIEWER (self->priv->viewer));
 	}
 
-	_gth_image_preloader_init_preloader (self);
-	gth_image_preloader_load (self->priv->preloader,
-				  self->priv->file_data,
+	_gth_image_viewer_page_load_with_preloader (self,
+						    self->priv->file_data,
 #ifdef ALWAYS_LOAD_ORIGINAL_SIZE
-				  GTH_ORIGINAL_SIZE,
+						    GTH_ORIGINAL_SIZE,
 #else
-				  _gth_image_preloader_get_requested_size_for_next_images (self),
+						    _gth_image_preloader_get_requested_size_for_next_images (self),
 #endif
-				  NULL,
-				  preloader_load_ready_cb,
-				  self,
-				  N_FORWARD_PRELOADERS + N_BACKWARD_PRELOADERS,
-				  next_file_data[0],
-				  next_file_data[1],
-				  prev_file_data[0],
-				  prev_file_data[1]);
-
+						    NULL,
+						    preloader_load_ready_cb,
+						    self);
 }
 
 
@@ -1726,6 +1852,7 @@ static void
 gth_image_viewer_page_finalize (GObject *obj)
 {
 	GthImageViewerPage *self;
+	int                 i;
 
 	self = GTH_IMAGE_VIEWER_PAGE (obj);
 
@@ -1747,6 +1874,11 @@ gth_image_viewer_page_finalize (GObject *obj)
 	_g_object_unref (self->priv->file_data);
 	_g_object_unref (self->priv->last_loaded);
 
+	for (i = 0; i < N_FORWARD_PRELOADERS; i++)
+		_g_clear_object (&self->priv->next_file_data[i]);
+	for (i = 0; i < N_BACKWARD_PRELOADERS; i++)
+		_g_clear_object (&self->priv->prev_file_data[i]);
+
 	G_OBJECT_CLASS (gth_image_viewer_page_parent_class)->finalize (obj);
 }
 
@@ -1793,6 +1925,7 @@ gth_image_viewer_page_init (GthImageViewerPage *self)
 	self->priv->file_popup_merge_id = 0;
 	self->priv->history = gth_image_history_new ();
 	self->priv->file_data = NULL;
+	self->priv->updated_info = NULL;
 	self->priv->active = FALSE;
 	self->priv->image_changed = FALSE;
 	self->priv->loading_image = FALSE;
@@ -1807,6 +1940,11 @@ gth_image_viewer_page_init (GthImageViewerPage *self)
 	self->priv->pointer_on_viewer = FALSE;
 	self->priv->hide_overview_id = 0;
 	self->priv->apply_icc_profile = TRUE;
+
+	for (i = 0; i < N_FORWARD_PRELOADERS; i++)
+		self->priv->next_file_data[i] = NULL;
+	for (i = 0; i < N_BACKWARD_PRELOADERS; i++)
+		self->priv->prev_file_data[i] = NULL;
 }
 
 
@@ -2034,6 +2172,7 @@ get_original_data_free (OriginalImageData *data)
 	if (data == NULL)
 		return;
 
+	_g_object_unref (data->viewer_page);
 	_g_object_unref (data->cancellable);
 	_g_object_unref (data->result);
 	g_free (data);
@@ -2046,8 +2185,13 @@ original_image_ready_cb (GObject	*source_object,
 			 gpointer	 user_data)
 {
 	OriginalImageData *data = user_data;
-	GthImage        *image = NULL;
-	GError          *error = NULL;
+	GthImage          *image = NULL;
+	GError            *error = NULL;
+
+	if (! _gth_image_viewer_page_load_with_preloader_finish (data->viewer_page)) {
+		get_original_data_free (data);
+		return;
+	}
 
 	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
 					       result,
@@ -2079,7 +2223,7 @@ gth_image_viewer_page_get_original (GthImageViewerPage	 *self,
 	OriginalImageData *data;
 
 	data = get_original_data_new ();
-	data->viewer_page = self;
+	data->viewer_page = g_object_ref (self);
 	data->result = g_simple_async_result_new (G_OBJECT (self),
 						  ready_callback,
 						  user_data,
@@ -2098,15 +2242,12 @@ gth_image_viewer_page_get_original (GthImageViewerPage	 *self,
 		get_original_data_free (data);
 	}
 	else {
-		_gth_image_preloader_init_preloader (self);
-		gth_image_preloader_load (self->priv->preloader,
-					  self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
-					  GTH_ORIGINAL_SIZE,
-					  data->cancellable,
-					  original_image_ready_cb,
-					  data,
-					  GTH_NO_PRELOADERS,
-					  NULL);
+		_gth_image_viewer_page_load_with_preloader (self,
+							    self->priv->image_changed ? GTH_MODIFIED_IMAGE : self->priv->file_data,
+							    GTH_ORIGINAL_SIZE,
+							    data->cancellable,
+							    original_image_ready_cb,
+							    data);
 	}
 }
 
