@@ -962,14 +962,14 @@ _g_utf8_strndup (const char *str,
 }
 
 
-static const char *
+const char *
 _g_utf8_strstr (const char *haystack,
 		const char *needle)
 {
 	const char *s;
-	gsize       i;
-	gsize       haystack_len = g_utf8_strlen (haystack, -1);
-	gsize       needle_len = g_utf8_strlen (needle, -1);
+	glong       i;
+	glong       haystack_len = g_utf8_strlen (haystack, -1);
+	glong       needle_len = g_utf8_strlen (needle, -1);
 	int         needle_size = strlen (needle);
 
 	s = haystack;
@@ -2952,6 +2952,36 @@ _g_file_info_get_secondary_sort_order (GFileInfo  *info)
 }
 
 
+void
+_g_file_info_update (GFileInfo  *dest_info,
+		     GFileInfo  *src_info)
+{
+	char **attributes;
+	int    i;
+
+	if (src_info == NULL)
+		return;
+
+	attributes = g_file_info_list_attributes (src_info, NULL);
+	for (i = 0; attributes[i] != NULL; i++) {
+		GFileAttributeType   type;
+		gpointer             value_pp;
+		GFileAttributeStatus status;
+
+		if (g_file_info_get_attribute_data (src_info,
+						    attributes[i],
+						    &type,
+						    &value_pp,
+						    &status))
+		{
+			g_file_info_set_attribute (dest_info, attributes[i], type, value_pp);
+		}
+	}
+
+	g_strfreev (attributes);
+}
+
+
 const char *
 _g_content_type_guess_from_name (const char *filename)
 {
@@ -3264,21 +3294,21 @@ _g_format_duration_for_display (gint64 msecs)
         if (hour > 0)
         {
                 /* hour:minutes:seconds */
-                /* Translators: This is a time format, like "9:05:02" for 9
-                 * hours, 5 minutes, and 2 seconds. You may change ":" to
+                /* Translators: This is a time format, like "9∶05∶02" for 9
+                 * hours, 5 minutes, and 2 seconds. You may change "∶" to
                  * the separator that your locale uses or use "%Id" instead
                  * of "%d" if your locale uses localized digits.
                  */
-                return g_strdup_printf (C_("long time format", "%d:%02d:%02d"), hour, min, sec);
+                return g_strdup_printf (C_("long time format", "%d∶%02d∶%02d"), hour, min, sec);
         }
 
         /* minutes:seconds */
-        /* Translators: This is a time format, like "5:02" for 5
-         * minutes and 2 seconds. You may change ":" to the
+        /* Translators: This is a time format, like "5∶02" for 5
+         * minutes and 2 seconds. You may change "∶" to the
          * separator that your locale uses or use "%Id" instead of
          * "%d" if your locale uses localized digits.
          */
-        return g_strdup_printf (C_("short time format", "%d:%02d"), min, sec);
+        return g_strdup_printf (C_("short time format", "%d∶%02d"), min, sec);
 }
 
 
