@@ -25,7 +25,6 @@
 #include <gtk/gtk.h>
 #include "glib-utils.h"
 
-DEF_ACTION_CALLBACK (toggle_action_activated)
 DEF_ACTION_CALLBACK (gth_application_activate_new_window)
 DEF_ACTION_CALLBACK (gth_application_activate_preferences)
 DEF_ACTION_CALLBACK (gth_application_activate_show_shortcuts)
@@ -60,5 +59,6 @@ DEF_ACTION_CALLBACK (gth_browser_activate_show_sidebar)
 DEF_ACTION_CALLBACK (gth_browser_activate_show_thumbnail_list)
 DEF_ACTION_CALLBACK (gth_browser_activate_show_previous_image)
 DEF_ACTION_CALLBACK (gth_browser_activate_show_next_image)
+DEF_ACTION_CALLBACK (gth_browser_activate_apply_editor_changes)
 
 #endif /* GTH_BROWSER_ACTIONS_CALLBACK_H */
